package de.hft.stuttgart.citydoctor2.gui.logger;

import java.io.Serializable;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.Property;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginElement;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;
import org.apache.logging.log4j.core.layout.PatternLayout;

import javafx.application.Platform;
import javafx.scene.control.TextArea;

@Plugin(name="GuiLogger", category="Core", elementType="appender", printObject=true)
public class GuiLogger extends AbstractAppender {
	
	private static TextArea area;
	
	private final ReadWriteLock rwLock = new ReentrantReadWriteLock();
	private final Lock readLock = rwLock.readLock();
	
	private static final int CAPACITY = 20000;
	
	private final StringBuilder buffer = new StringBuilder(CAPACITY);
	private Thread daemonLoggerThread;
	private boolean dirty = false;
	
	public static void setTextArea(TextArea area) {
		GuiLogger.area = area;
	}
	
	@PluginFactory
    public static GuiLogger createAppender(
            @PluginAttribute("name") String name,
            @PluginAttribute("ignoreExceptions") boolean ignoreExceptions,
            @PluginElement("Layout") Layout<? extends Serializable> layout,
            @PluginElement("Filter") final Filter filter) {
        if (name == null) {
            LOGGER.error("No name provided for GuiLogger");
            return null;
        }
        if (layout == null) {
            layout = PatternLayout.createDefaultLayout();
        }
        GuiLogger guiLogger = new GuiLogger(name, filter, layout, ignoreExceptions, new Property[0]);
        guiLogger.daemonLoggerThread = new Thread(() -> {
			while (!Thread.interrupted()) {
				if (guiLogger.dirty) {
					Platform.runLater(() -> {
						// set text doesn't scroll
						// workaround with clear -> append
						area.clear();
						area.appendText(guiLogger.buffer.toString());
					});
					guiLogger.dirty = false;
				}
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {
					Thread.currentThread().interrupt();
				}
			}
		});
        guiLogger.daemonLoggerThread.setDaemon(true);
        guiLogger.daemonLoggerThread.start();
        return guiLogger;
    }
	
	protected GuiLogger(String name, Filter filter, Layout<? extends Serializable> layout, boolean ignoreExceptions,
			Property[] properties) {
		super(name, filter, layout, ignoreExceptions, properties);
	}

	@Override
	public void append(LogEvent event) {
		if (area == null) {
			return;
		}
		readLock.lock();
		try {
			String s = new String(getLayout().toByteArray(event));
			int capacityLeft = CAPACITY - buffer.length();
			if (capacityLeft < s.length()) {
				int delete = s.length() - capacityLeft;
				buffer.delete(0, delete - 1);
			}
			buffer.append(s);
			dirty = true;
		} finally {
			readLock.unlock();
		}
	}
	
	

}
