package de.hft.stuttgart.citydoctor2.healer.genetic;

public class Individual<T> implements Comparable<Individual<T>> {
	
	private T object;
	private double fitness;
	
	public Individual(T object) {
		this.object = object;
	}
	
	public void evaluateFitness(FitnessEvaluation<T> evaluation) {
		fitness = evaluation.evaluate(object);
	}
	
	public double getFitness() {
		return fitness;
	}

	@Override
	public int compareTo(Individual<T> o) {
		return Double.compare(o.getFitness(), fitness);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(fitness);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		result = prime * result + ((object == null) ? 0 : object.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Individual<?> other = (Individual<?>) obj;
		if (Double.doubleToLongBits(fitness) != Double.doubleToLongBits(other.fitness)) {
			return false;
		}
		if (object == null) {
			if (other.object != null) {
				return false;
			}
		} else if (!object.equals(other.object)) {
			return false;
		}
		return true;
	}

	public T getObject() {
		return object;
	}

	@Override
	public String toString() {
		return "Genotype [object=" + object + ", fitness=" + fitness + "]";
	}

	public void setFitness(double fitness) {
		this.fitness = fitness;
	}

}
